import { Selector, userVariables } from 'testcafe';
import { AppsPage, getAdminUser } from '../common.models';
import {
  AddConnect4,
  AddCore3,
  AddDIO,
  AddMIO,
  AddRO,
  clickMenu,
  DeviceCatalog,
  DeviceIds,
  Editor,
  FileMenu,
  ValueEditor,
} from './pictory.models';
import {
  backupPictoryProjectsDir,
  removePictoryProjectsBackupDir,
  restorePictoryProjectsDir,
  validatePictoryStartupConfig,
} from './utils';

fixture(`Pictory Device Handling`)
  .before(async () => {
    // Backup the pictory projects directory before starting the tests
    await backupPictoryProjectsDir();
  })
  .after(async () => {
    // Remove the back up of pictory projects directory after finishing all tests
    await removePictoryProjectsBackupDir();
  })
  .afterEach(async () => {
    // After each test, restore the pictory projects directory to revert all changes made during the test
    await restorePictoryProjectsDir();
  })
  .beforeEach(async (t) => {
    // Begin each test with:

    // 1. login into webstatus
    await t.useRole(
      getAdminUser(
        userVariables.targetUrl as string,
        userVariables.targetPassword as string,
      ),
    );

    // 2. Open pictory
    await t.click(AppsPage.pictoryButton);
    await t.wait(1000);

    // 3. Check that we start with an empty pictory
    await t.expect(Editor.AllDevices.count).eql(0);
  });

async function reloadCurrentPage(t: TestController) {
  await t.setNativeDialogHandler((dialogType, message, url) => {
    if (dialogType === 'beforeunload') return;

    throw Error(
      `An unexpected ${dialogType} dialog with the message "${message}" appeared on ${url}.`,
    );
  });

  const currentUrl = await t.eval(() => document.documentURI);
  await t.navigateTo(currentUrl);
}

test('Add Core3 Base Device with MIO+DIO and save as start config', async (t) => {
  await AddCore3();
  await AddDIO();
  await AddMIO();

  await clickMenu(t, FileMenu, 'SaveAsStart');
  await reloadCurrentPage(t);

  // Assert devices are still there after reloading the page
  await t
    .expect(Editor.BaseDevice.exists)
    .ok()
    .expect(Editor.BaseDevice.id)
    .contains(DeviceIds.Core3)

    .expect(Editor.CurrentDevices.DIO.exists)
    .ok()
    .expect(Editor.LeftDevice.id)
    .contains(DeviceIds.DIO)

    .expect(Editor.CurrentDevices.MIO.exists)
    .ok()
    .expect(Editor.RightDevice.id)
    .contains(DeviceIds.MIO);

  const result = await validatePictoryStartupConfig();
  await t.expect(result.isValid).ok(result.error);
});

test('Add Connect4 with RO and save as named project and load it again', async (t) => {
  await AddConnect4();
  await AddRO();
  await clickMenu(t, FileMenu, 'SaveAs');

  await t
    .typeText('#txtInputFilename', 'connect4_ro')
    .click(Selector('#okDialogSaveProject span').withText('Ok'));

  await clickMenu(t, FileMenu, 'Clear');

  await t.expect(Editor.AllDevices.count).eql(0);

  await clickMenu(t, FileMenu, 'Open');

  await t
    .expect(Selector('#tblProjectFiles b').withText('connect4_ro').exists)
    .ok();
  await t.click(Selector('#dlgSaveProjectOpen_connect4_ro').withText('Open'));
});

test('Add Connect4 with DIO and set custom names/values for Input and Outputs', async (t) => {
  await AddConnect4();
  await AddDIO();

  // Value Table should be visible
  await t.expect(ValueEditor.Table.exists).ok();

  // Row #1 should contain DIO's Input 1
  const firstInput = ValueEditor.getColumnsOfRow(0);
  await t.expect(firstInput.Type.innerText).eql('INP');
  //  Set custom name
  const input1Name = firstInput.Name.find('input');
  await t.expect(input1Name.value).eql('I_1');
  await t
    .selectText(input1Name)
    .pressKey('delete')
    .typeText(input1Name, 'CustomInput1')
    .pressKey('enter');

  // Row #33 should contain DIO's Output 1
  const firstOutput = ValueEditor.getColumnsOfRow(34);
  // First row's label should be type INPUT
  await t.expect(firstOutput.Type.innerText).eql('OUT');
  //  Set custom name
  const output1Name = firstOutput.Name.find('input');
  await t.expect(output1Name.value).eql('O_1');
  await t
    .selectText(output1Name)
    .pressKey('delete')
    .typeText(output1Name, 'CustomOutput1')
    .pressKey('enter');

  // Save and reload
  await clickMenu(t, FileMenu, 'SaveAsStart');
  await reloadCurrentPage(t);

  // Click DIO to open value editor
  await t.click(Editor.CurrentDevices.DIO);

  // Expect custom names still to be there
  await t.expect(input1Name.value).eql('CustomInput1');
  await t.expect(output1Name.value).eql('CustomOutput1');

  const result = await validatePictoryStartupConfig();
  await t.expect(result.isValid).ok(result.error);
});

test('Adding a Gateway left to Connect4 should show error message', async (t) => {
  await AddConnect4();

  await t.dragToElement(
    DeviceCatalog.Gateways.Gateway_DMX,
    Editor.DropTargetEdgeLeft,
  );

  await t
    .expect(Selector('#lblDialogInfo').innerText)
    .contains('ERR_LEFT_INVALID_PRODUCTTYPE');
});
