import { join } from 'path';
import { access, readFile } from 'node:fs/promises';
import { exec, execFile } from 'node:child_process';
import { promisify } from 'node:util';

const execP = promisify(exec);

export const isLocal =  process.env['TEST_PICTORY_PATH'] !== undefined;
export const pictoryPath =
  process.env['TEST_PICTORY_PATH'] ?? '/var/www/revpi/pictory';
const pictoryProjectPath = join(pictoryPath, 'projects');
const pictoryProjectBackupPath = join(pictoryPath, 'projects_backup');



/**
 * Backs up the original Pictory projects directory.
 * @returns {Promise<void>} A promise that resolves when the backup is complete.
 * @throws {Error} If the Pictory projects directory does not exist.
 */
export async function backupPictoryProjectsDir (): Promise<void> {
  try {
    await access(pictoryProjectPath);
  } catch (error) {
    throw new Error(
      `Could not find pictory projects dir at ${pictoryProjectPath}. ${error}`,
    );
  }

  await removePictoryProjectsBackupDir();
  await runCommand(`cp -a ${pictoryProjectPath} ${pictoryProjectBackupPath}`);
}

/**
 * Restores the Pictory Projects directory by removing the existing directory and copying the backup directory to its original location.
 *
 * @returns {Promise<void>} A Promise that resolves when the restoration is complete or rejects if an error occurs.
 */
export async function restorePictoryProjectsDir (): Promise<void> {
  await runCommand(`rm -rf ${pictoryProjectPath}`);
  await runCommand(`cp -a ${pictoryProjectBackupPath} ${pictoryProjectPath}`);
}

/**
 * Removes the backup directory for Pictory projects.
 * @returns Promise<void>
 */
export async function removePictoryProjectsBackupDir (): Promise<void> {
  await runCommand(`rm -rf ${pictoryProjectBackupPath}`);
}

export async function runCommand (command: string): Promise<void> {
  const { stdout, stderr } = await execP(command);
  if (stderr) {
    console.error(`Command '${command}' failed)`);
    console.error(stderr);
    throw new Error(`Command '${command}' failed)`);
  }

  console.debug(`Command '${command}' succeeded)`);
  console.debug(stdout);
}

type ValidationResult = { isValid: boolean; error?: string };

export async function validatePictoryStartupConfig (): Promise<ValidationResult> {
  // If the tests are executed locally the revpimodio script will not work so we just return true
  if(isLocal) {
    return Promise.resolve({ isValid: true })
  }

  return new Promise<ValidationResult>((resolve) => {
    const scriptPath = join(process.cwd(), 'tests/validate-start-config.py');

    execFile(
      '/usr/bin/python3',
      [scriptPath],
      { encoding: 'buffer' },
      (error, stdout, stderr) => {
        if (error !== null) {
          return resolve({ isValid: false, error: error.message });
        }

        if (stderr?.length !== 0) {
          return resolve({ isValid: false, error: stderr.toString() });
        }

        resolve({ isValid: true });
      },
    );
  });
}

